/// @addtogroup bitSet
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file BitSetT.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Marian Mrozek 2005-2006
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.





#ifndef _BITSET_H_
#define _BITSET_H_
#include <iostream>


template <typename P_Bitmap>
class BitSetT : public P_Bitmap{
protected:

public:

  using P_Bitmap::Length;
  typedef typename P_Bitmap::Word Word;
  typedef typename P_Bitmap::BitIterator BitIterator;
  typedef typename P_Bitmap::WordIterator WordIterator;
  typedef P_Bitmap BaseClass;

  class PointIterator;
  explicit BitSetT(int A_wordLength=0, bool A_clear=false):P_Bitmap(A_wordLength,A_clear){}   // BitSetT with A_wordLength words
  explicit BitSetT(int A_wordLength, const char* bytes):P_Bitmap(A_wordLength,bytes){}        // BitSetT with A_wordLength words, using a provided contents
  explicit BitSetT(const std::string& A_s):P_Bitmap(A_s){}   // BitSetT with A_wordLength words filled with data from a string
  BitSetT(const P_Bitmap& A_org, bool A_clear=false):P_Bitmap(A_org,A_clear){}
  const BaseClass& getBaseObject() const{ return *static_cast<const BaseClass*>(this);}

  friend class PointIterator;

  int cardinality() const;

  friend void swap(BitSetT& A_BitSet1,BitSetT& A_BitSet2){
    swap(static_cast<P_Bitmap &>(A_BitSet1),static_cast<P_Bitmap &>(A_BitSet2));
  }
};

/************************************************************************************/

#include "bitSet/BitSet_PointIterator.h"

template <typename P_Bitmap>
inline int BitSetT<P_Bitmap>::cardinality() const{
  int cardi=0;
  PointIterator eIt=this->end();
  for(PointIterator it=this->begin();it<eIt;++it){
    ++cardi;
  }
  return cardi;
}

#endif

/// @}

