/////////////////////////////////////////////////////////////////////////////
/// @addtogroup Interval
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file mpRounding.h
///
/// Functions that change rounding modes and precisions 
/// of Multi Precision Interval Arithmetic
///
/// @author Tomasz Kapela   @date 29-08-2006
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Tomasz Kapela 2006 
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "multiPrec/mpSetting.h"
#ifdef __HAVE_MPFR__

#ifndef _MP_ROUNDING_H_
#define _MP_ROUNDING_H_

#include "multiPrec/mpfrClass.h"

namespace capd{
namespace multiPrec{
   
/////////////////////////////////////////////////////////////////////////////
// roundUp
///
///  sets rounding up mode 
/////////////////////////////////////////////////////////////////////////////
inline void roundUp()
{
  MpfrClass::setDefaultRndMode(RoundUp);    
}

/////////////////////////////////////////////////////////////////////////////
// roundDown
///
///  sets rounding down mode 
/////////////////////////////////////////////////////////////////////////////
inline void roundDown()
{
  MpfrClass::setDefaultRndMode(RoundDown);    
}

/////////////////////////////////////////////////////////////////////////////
// roundNearest
///
///  sets rounding to the nearest mode 
/////////////////////////////////////////////////////////////////////////////
inline void roundNearest()
{
  MpfrClass::setDefaultRndMode(RoundNearest);    
}

/////////////////////////////////////////////////////////////////////////////
// setPrecision
///
///  sets default precision for all multi precision operations
///
///  @param[in]  newprec   new precision (number of mantisa bits)
///
/////////////////////////////////////////////////////////////////////////////
inline void setPrecision(PrecisionType newprec)
{
   MpfrClass::setDefaultPrecision(newprec);  
}

/////////////////////////////////////////////////////////////////////////////
// getPrecision
///
///  @returns default precision for all multi precision operations
///
/////////////////////////////////////////////////////////////////////////////
inline PrecisionType getPrecision()
{
   return MpfrClass::getDefaultPrecision();  
}

}}  // end of namespace capd::multiPrec

#endif // _MP_ROUNDING_H_

#endif  // __HAVE_MPFR__

/// @}
