/// @addtogroup multiEngHom
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file CrHomMD.cpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2005-2007 by Marian Mrozek
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <new>
using namespace std;

#include "homologicalAlgebra/CrHomMD.h"
#include "auxil/handleUnexpectedTerminate.h"
#include "auxil/commandLineArgs.h"

int main(int argc,char** argv){
  set_unexpected(handle_unexpected);
  set_terminate(handle_terminate);
  fcout.open("report.txt");
  fcout.turnOn();

  setupCommandLineArgs;
  declareCommandLineArg(std::string,inFile,"");
  declareCommandLineArg(std::string,engine,"CR");
  declareCommandLineArg(std::string,help,"");
  declareCommandLineArg(int,verbose,2);
  std::string executableName("CrHomMD");
  std::string version(" 0.9 ");
  std::string copyright("(c) Marian Mrozek, 2007");
  std::string logo(executableName+version+copyright);

  std::pair<int,std::string> vc= getValueCountAndArg(std::string("1"));
  if(vc.first>1){
    fcout << "Only one file can be processed\n";
    exit(1);
  }else if(vc.first==1){
    if(vc.second=="help") help="options";
    else inFile=vc.second;
  }else if(vc.first==0 && help==""){
    help="intro";
  }

  if(help != ""){
    if(help=="intro"){
      fcout << logo  << std::endl;
      fcout << "Basic usage: \n";
      fcout << "      " << executableName << " filename\n";
      fcout << "Type: \n";
      fcout << "      " << executableName << " help=xx\n";
      fcout << "where xx may be:  input  or output\n";
      fcout << "to learn about available input/output formats\n";
      exit(0);
    }else if(help=="input"){
      fcout << logo  << std::endl;
      fcout << "" << executableName << " accepts files in the following formats\n";
      fcout << "   bmp - two dimensional bitmaps in the standard BMP bitmap format\n";
      fcout << "   bmd - multidimensional bitmaps in the bmd format\n";
      fcout << "   cub/txt - various text formats\n";
      fcout << "Consult http://ii.uj.edu.pl/~mrozek/software/homology.html for details\n";
      exit(0);
    }else if(help=="output"){
      fcout << logo  << std::endl;
      fcout << "The amount of information provided may be controlled with  \n\n";
      fcout << "   verbose=n\n\n";
      fcout << "where n means the following output\n";
      fcout << "   0 - only Betti numbers\n";
      fcout << "   1 - Betti numbers and computation time \n";
      fcout << "   2 - Betti and torsion numbers (default)\n";
      fcout << "   3 - Betti and torsion numbers with some details of the computation process\n";
      fcout << "Consult http://ii.uj.edu.pl/~mrozek/software/homology.html for details\n";
      exit(0);
    }
  }

  try{
    CrHom<0> ch;
    ch(inFile.c_str(),engine,verbose);
  }catch(std::exception& e){
    fcout.turnOn();
    fcout << "Caught exception: " << e.what() << endl;
  }catch(std::string& s){
    fcout.turnOn();
    fcout << "Caught exception: " << s.c_str() << endl;
  }catch(const char* c){
    fcout.turnOn();
    fcout << "Caught exception: " << c << endl;
  }catch(...){
    fcout.turnOn();
    fcout << "Caught an unknown exception: " << endl;
  }

}

/// @}

